<?php
namespace Util\Mail;
use Teknisa\Libs\Util\Utilities;
use Teknisa\Model\Teknisa\Entities\Logfasetram;
use Util\InstanceProvider;
use Util\Mail\Core\SmtpMail;
use Util\Messages;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zend\Mime\Part as ZendMimePart;
use Zend\Mime\Message as ZendMimeMessage;

class SoliservMail {

    /**
     * @var \Doctrine\ORM\EntityManager
     */
    private $entityManager;
    /**
     * @var SmtpMail
     */
    private $smtpMail;

    private $baseDir ='';

    public function __construct()
    {
        $this->entityManager = InstanceProvider::getEntityManager();
        $this->smtpMail = InstanceProvider::getSmtpMail();
        $this->baseDir = __DIR__.DIRECTORY_SEPARATOR.'..'.DIRECTORY_SEPARATOR.'EmailTemplates'.DIRECTORY_SEPARATOR;
    }
    public function sendEmailGep(\DateTime $commentDate,$callRelationship){
        $commentDateString = $commentDate->format('dmYHis');
        $emails = InstanceProvider::getRepository()->getSoliservLocal()->getUnsentEmails($commentDateString,$callRelationship);
        foreach($emails as $i=>$email){
            $message = $this->getEmailMessage($email);
            try {
                $this->encodeAndSend($email["DSEMAILOPER"], "Solicitação ".$email["DSREFESOLI"], $message);
                $this->updateEmailEnvProductCallComment($email["CDFILIAL"],$email["NRSOLISERV"], new \DateTime(str_replace("/","-",$email["DTALFASETRAM"])));
            }catch(\Exception $e){
            }
        }
    }

    private function encodeAndSend($to, $subject, $message)
    {
        $message        = utf8_decode($message);
        $mimePart       = new ZendMimePart($message);
        $mimeMessage    = new ZendMimeMessage();
        $mimePart->type = 'text/html';
        $mimeMessage->setParts(array($mimePart));
        $this->sendHtml($to, $subject, $mimeMessage);
    }

    private function getEmailMessage($email){
        $message = $this->fileGetContentUtf8($this->baseDir . 'mailGep.html');
        $message = str_replace("%title%",$email["DSREFESOLI"],$message);
        $message = str_replace("%name%",$email["NMOPERADOR"],$message);
        $message = str_replace("%left%",$email["SOLI"],$message);
        $message = str_replace("%comment%",
            empty($email["DSALFASETRAM"]) ? "" :
                "<tr>
                <td class='title'>
                    <div>Comentário</div>
                </td>
            </tr>
            <tr>
                <td class='description'>
                    <div class='divTable'>
                        <div >".$email["DSALFASETRAM"]."</div>
                    </div>
                </td>
            </tr>"
            ,$message);
        $message = str_replace("%date%",$email["DTALFASETRAM"],$message);
        $message = str_replace("%action%",$email["TXACAOLFTRAM"],$message);
        return $message;
    }

    private function updateEmailEnvProductCallComment($cdfilial, $nrsoliserv, $dtalfasetram){
        $connection = $this->entityManager->getConnection();
        try {
            $connection->beginTransaction();

            $logfaseTramRepository    = $this->entityManager->getRepository('\Teknisa\Model\Teknisa\Entities\Logfasetram');
            /** @var Logfasetram $log */
            $log = $logfaseTramRepository->findOneBy(array('cdfilial' => $cdfilial,'nrsoliserv' => $nrsoliserv,'dtalfasetram' => $dtalfasetram));
            $logFaseTramFactory = InstanceProvider::getFactory()->getLogFaseTram();
            $log = $logFaseTramFactory->updateEmailEnv($log);
            $this->entityManager->persist($log);
            $this->entityManager->flush();
            $connection->commit();

        } catch(\Exception $e) {
            $connection->rollBack();
            throw $e;
        }

    }

    private function sendHtml($to, $subject, $message)
    {
        $this->smtpMail->setFrom("pm@teknisa.com");
        $this->smtpMail->setTo($to);
        $this->smtpMail->setSubject($subject);
        $this->smtpMail->setBody($message);
        $this->smtpMail->sendHtml();
    }

    private function fileGetContentUtf8($fileName)
    {
        $content = file_get_contents($fileName);
        return mb_convert_encoding($content, 'UTF-8', mb_detect_encoding($content, 'UTF-8, ISO-8859-1', true));
    }
}
