/*
   Descrio: Encaixa a data de entrega em uma data de produo, verificando a periodicidade e o calendario da produo.
   Criao: Welbert G. Moitinha
   Data: 28/06/2010

*/


IF EXISTS(SELECT * FROM sysobjects O, sysusers U
          WHERE
            o.name = 'BUSCA_DATA_PRODUCAO'  and
            type   = 'P'                 and
            u.uid  = o.uid               and
            u.name = USER)
  DROP PROCEDURE BUSCA_DATA_PRODUCAO
GO


CREATE PROCEDURE BUSCA_DATA_PRODUCAO @P_CDFILIAL VARCHAR(4),
                                     @P_CDPRODUTO VARCHAR(10),
                                     @P_CDPERIPRODU VARCHAR(2),
                                     @P_NRTOTLOTEDIA  INT,
                                     @P_DTMRP DATETIME,
                                     @P_DTPRODUCAO DATETIME,
                                     @P_QTAFABRICDIA DOUBLE PRECISION,
                                     @P_DTATUAL DATETIME,
                                     @P_GERACALE VARCHAR(1) OUTPUT,
                                     @P_DTVALIDPROD DATETIME OUTPUT

AS
BEGIN
   DECLARE @V_NRLOTESDIAS INT;
   DECLARE @V_GERACALE VARCHAR(1);
   DECLARE @V_DTAPRODUZIR DATETIME;
END
GO

ALTER PROCEDURE BUSCA_DATA_PRODUCAO @P_CDFILIAL VARCHAR(4),
                                    @P_CDPRODUTO VARCHAR(10),
                                    @P_CDPERIPRODU VARCHAR(2),
                                    @P_NRTOTLOTEDIA INTEGER,
                                    @P_DTMRP DATETIME,
                                    @P_DTPRODUCAO DATETIME,
                                    @P_QTAFABRICDIA DOUBLE PRECISION,
                                    @P_DTATUAL DATETIME,
                                    @P_GERACALE VARCHAR(1) OUTPUT,
                                    @P_DTVALIDPROD DATETIME OUTPUT
AS
BEGIN
   DECLARE @V_NRLOTESDIAS INT;
   DECLARE @V_GERACALE VARCHAR(1);
   DECLARE @V_DTAPRODUZIR DATETIME;

/* VERIFICA SE A DATA SUGERIDA A PRODUZIR PERTENCE AO CALENDARIO DE PRODUCAO */
   SET @V_GERACALE = 'S';

   SELECT
      @V_DTAPRODUZIR = MAX(DTPRODUCAO)
     FROM
      CALEPRODUCAO
    WHERE
      (CDFILIAL             = @P_CDFILIAL)             AND
      (CDPERIPRODU          = @P_CDPERIPRODU)  AND
      (CONVERT(VARCHAR, DTPRODUCAO, 103) <= CONVERT( VARCHAR, @P_DTPRODUCAO, 103));

   IF NOT @V_DTAPRODUZIR IS NULL BEGIN
/*VERIFICA SE D PRA CONSOLIDAR A QUANTIDADE DE PRODUO DO MESMO DIA DENTRO DE UM LOTE
VERIFICANDO A QUANTIDADE REAL DE PRODUCAO SEM ARREDONDAMENTO*/

      SET @V_DTAPRODUZIR = (@V_DTAPRODUZIR + (@P_DTPRODUCAO - CONVERT(DATETIME, @P_DTPRODUCAO, 103) ));
/* ENCAIXA A HORA NA PRODUO */
      SET @V_DTAPRODUZIR = TEKNISA.ENCAIXA_PRODUCAO_TURNO( @P_CDFILIAL , @V_DTAPRODUZIR);

      EXEC VERIFICA_SALDO_LOTE_PROD  @P_CDFILIAL, @P_CDPRODUTO, @P_DTMRP, @V_DTAPRODUZIR, @P_QTAFABRICDIA, @V_GERACALE;

      SET @P_GERACALE = @V_GERACALE;

      IF(@V_GERACALE = 'S')BEGIN

         SELECT
            @V_NRLOTESDIAS = COUNT(*)
         FROM
            CALEPRODMRP
         WHERE
            (CDFILIAL              = @P_CDFILIAL)          AND
            (DTPROGPRODCP          = @P_DTMRP)             AND
            (CDPRODUTO             = @P_CDPRODUTO) AND
            ( CONVERT(DATETIME, DTCONCPROMRP, 103) =  CONVERT( DATETIME, @V_DTAPRODUZIR, 103));

         IF (@V_NRLOTESDIAS = @P_NRTOTLOTEDIA) BEGIN
            SET @V_DTAPRODUZIR = @V_DTAPRODUZIR - 1;

            EXEC BUSCA_DATA_PRODUCAO @P_CDFILIAL, @P_CDPRODUTO, @P_CDPERIPRODU, @P_NRTOTLOTEDIA, @P_DTMRP, @V_DTAPRODUZIR, @P_QTAFABRICDIA, @P_DTATUAL, @V_GERACALE, @V_DTAPRODUZIR;

            IF @V_DTAPRODUZIR < @P_DTATUAL BEGIN
               SET @P_GERACALE = 'M';
               SET @P_DTVALIDPROD = NULL;
            END
            ELSE BEGIN
               SET @P_GERACALE = @V_GERACALE;
               SET @P_DTVALIDPROD = @V_DTAPRODUZIR;
            END;
         END
         ELSE BEGIN
            SET @P_GERACALE = @V_GERACALE;
            SET @P_DTVALIDPROD =@V_DTAPRODUZIR;
         END;
      END
      ELSE BEGIN
         SET @P_DTVALIDPROD =@V_DTAPRODUZIR;
      END;
   END
   ELSE BEGIN
      SET @P_GERACALE = 'C';
      SET @P_DTVALIDPROD = NULL;
   END;
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'BUSCA_DATA_PRODUCAO','4.00.000';
GO

